package com.esignlive.app;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.TextInputEditText;
import android.util.Log;
import android.view.ActionMode;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

import com.esignlive.sdk.AuthenticationListener;
import com.esignlive.sdk.ESignLive;
import com.esignlive.sdk.datamodel.User;
import com.esignlive.sdk.utility.Error;

/**
 * A login screen that offers login via email/password.
 */
public class LoginActivity extends Activity implements OnClickListener {

    // UI references.
    private TextInputEditText loginEmailEditText;
    private TextInputEditText loginPasswordEditText;
    private TextInputEditText loginSessionTokenEditText;
    private Button loginWithEmailPasswordButton;
    private Button loginWithSessionTokenButton;
    private BlockingMessageView blockingMessageView;

    // Properties
    private boolean loginInProgreess = false;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (UserData.isUserLoggedIn()) {
            navigateToHomeActivity(this,true);
            return;
        }
        setContentView(R.layout.activity_login);
        this.getUIReferences();
        this.setupListeners();
        this.blockingMessageView.hide(false, null);
    }

    // UI components
    private void getUIReferences() {
        this.loginEmailEditText = (TextInputEditText) this.findViewById(R.id.loginEmailEditText);
        this.loginPasswordEditText = (TextInputEditText) this.findViewById(R.id.loginPasswordEditText);
        this.loginSessionTokenEditText = (TextInputEditText) this.findViewById(R.id.loginSessiontokenEditText);
        this.loginWithEmailPasswordButton = (Button) this.findViewById(R.id.loginWithEmailPasswordButton);
        this.loginWithSessionTokenButton  = (Button) this.findViewById(R.id.loginWithSessionTokenButton);
        this.blockingMessageView = (BlockingMessageView) this.findViewById(R.id.login_blockingMessageView);
    }
    private void setupListeners() {
        this.loginWithEmailPasswordButton.setOnClickListener(this);
        this.loginWithSessionTokenButton.setOnClickListener(this);
    }

    /**
     * Attempts to sign in or register the account specified by the login form.
     * If there are form errors (invalid email, missing fields, etc.), the
     * errors are presented and no actual login attempt is made.
     */
    private void attemptLoginWithEmailAndPassword() {
        // Reset errors.
        if (loginInProgreess){
            return;
        }
        loginEmailEditText.setError(null);
        loginPasswordEditText.setError(null);

        // Store values at the time of the login attempt.
        final String email = loginEmailEditText.getText().toString();
        final String password = loginPasswordEditText.getText().toString();

        boolean cancel = false;
        View focusView = null;

        // Check for a valid password, if the user entered one.
        if (password != null && !this.isPasswordValid(password)) {
            loginPasswordEditText.setError(getString(R.string.error_invalid_password));
            focusView = loginPasswordEditText;
            cancel = true;
        }

        // Check for a valid email address.
        if (email != null && !isEmailValid(email)) {
            loginEmailEditText.setError(getString(R.string.error_invalid_email));
            focusView = loginEmailEditText;
            cancel = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            if (focusView != null) {
                focusView.requestFocus();
                this.setupListeners();
            }
        } else {
            // Show a progress spinner, and kick off a background task to
            // perform the user login attempt.
            loginInProgreess = true;
            this.blockingMessageView.show("Login in progress", true, null);
            ESignLive.getInstance().login(email,password, new AuthenticationListener() {

                @Override
                public void loginSucceeded(User userInfo, boolean isOnlineLogin) {
                    LoginActivity.this.blockingMessageView.hide(true, null);
                    UserData.storeLoggedInUserDetails(email,password);
                    navigateToHomeActivity(LoginActivity.this,false);
                    loginInProgreess = false;
                }

                @Override
                public void loginFailed(Error error) {
                    LoginActivity.this.blockingMessageView.hide(true, null);
                    UserData.setUserNotLoggedIn();
                    loginInProgreess = false;
                    switch (error.getCode()) {
                        case Error.EMAIL_MISSING_ERROR:
                            LoginActivity.this.loginEmailEditText.setError(getString(R.string.error_field_required));
                            break;

                        case Error.PASSWORD_MISSING_ERROR:
                            LoginActivity.this.loginPasswordEditText.setError(getString(R.string.error_field_required));
                            break;

                        case Error.USER_NAME_PASSWORD_COMBINATION_ERROR:
                            Toast.makeText(LoginActivity.this, error.getMessage(), Toast.LENGTH_SHORT).show();
                            break;

                        default:
                            Toast.makeText(LoginActivity.this, error.getMessage(), Toast.LENGTH_SHORT).show();
                            break;
                    }
                    Log.e("APP",error.toString());
                }
            });
        }
    }
    private void attemptLoginWithSessionToken() {
        // Reset errors.
        if (this.loginInProgreess){
            return;
        }
        this.loginSessionTokenEditText.setError(null);

        // Store values at the time of the login attempt.
        final String sessionToken = this.loginSessionTokenEditText.getText().toString();

        this.loginInProgreess = true;
        this.blockingMessageView.show("Login in progress", true, null);
        ESignLive.getInstance().login(sessionToken, new AuthenticationListener() {
            @Override
            public void loginSucceeded(User user, boolean isOnlineLogin) {
                LoginActivity.this.blockingMessageView.hide(true, null);
                UserData.setUserLoggedIn(user.getEmail());
                navigateToHomeActivity(LoginActivity.this,false);
                loginInProgreess = false;
            }

            @Override
            public void loginFailed(Error error) {
                LoginActivity.this.blockingMessageView.hide(true, null);
                UserData.setUserNotLoggedIn();
                loginInProgreess = false;
                switch (error.getCode()) {
                    case Error.SESSION_TOKEN_MISSING_ERROR:
                        LoginActivity.this.loginSessionTokenEditText.setError(getString(R.string.error_field_required));
                        break;

                    default:
                        Toast.makeText(LoginActivity.this, error.getMessage(), Toast.LENGTH_SHORT).show();
                        break;
                }
                Log.e("APP",error.toString());
            }
        });
    }

    // Navigation
    public static void navigateToHomeActivity(Context context,boolean performLogin){
        Intent intent = new Intent(context, HomeActivity.class);
        intent.putExtra("performLogin",performLogin);
        context.startActivity(intent);
    }

    // Validations
    private boolean isEmailValid(String email) {
        //TODO: Replace this with your own logic
        return email.contains("@") && email.contains(".");
    }
    private boolean isPasswordValid(String password) {
        //TODO: Replace this with your own logic
        if (password == null) {
            return false;
        }
        return password.length() > 4;
    }

    // Interface implementation
    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.loginWithEmailPasswordButton:
                this.attemptLoginWithEmailAndPassword();
                break;

            case R.id.loginWithSessionTokenButton:
                this.attemptLoginWithSessionToken();
                break;

            default:
                break;
        }
    }

    // Activity LifeCycle
    @Override
    public void onActionModeFinished(ActionMode mode) {
        super.onActionModeFinished(mode);
        this.blockingMessageView.hide(false, null);
    }
    @Override
    public void onBackPressed() {
        super.onBackPressed();
        if (this.blockingMessageView != null) {
            this.blockingMessageView.hide(true, null);
        }
    }
}

